%Likelihood calculation - uses method from Calvet&Fisher 2004

function [LL,LLs,pi_mat] = FB_matlab_gamma(sigma,sigma_mixture,regime,RV_t,P,p_mixture,var_t,gamma_param,gamma_dist,model_variance)
if(nargin<10)
    model_variance=1; %%% Otherwise modelisation of the volatilities
    if(nargin<9)
        gamma_dist = 0;
    end
end


T = length(RV_t);                       
pi_mat = zeros(T+1,regime);     
LLs = zeros(1,T);
pi_mat(1,:) = (1/regime)*ones(1,regime);

%*----------------------------------------------------------------------*
%*                        Likelihood algorithm                          *
%*----------------------------------------------------------------------*
% pa = (2*pi)^-0.5;


if(model_variance==0)
    std_sigma = sqrt(sigma);
    s = repmat(std_sigma',T,1);
    if(sum(var_t)~=T)
        for q=1:regime
            s(:,q) = s(:,q).*sqrt(var_t);
        end
    end
    w_t = repmat(RV_t,1,regime);
    regime_mixture = max(size(sigma_mixture));
    std_sigma_mixture = sqrt(sigma_mixture);
    s_mixture = repmat(std_sigma_mixture',T,1);

else
    std_sigma = sigma;
    s = repmat(std_sigma',T,1);
    if(sum(var_t)~=T)
        for q=1:regime
            s(:,q) = s(:,q).*var_t;
        end
    end
    w_t = repmat(RV_t,1,regime);
    regime_mixture = max(size(sigma_mixture));
    std_sigma_mixture = sigma_mixture;
    s_mixture = repmat(std_sigma_mixture',T,1);
end


if(gamma_dist==0) %%% Inverse Gamma
    dist_param = [(gamma_param+1) gamma_param];
    eps = (w_t./s);
    pa = (dist_param(2)^dist_param(1)).*(eps).^(-dist_param(1)-1)./(s.*exp(gammaln(dist_param(1)))); %%% ./s comes from the gradient
    A = - dist_param(2)./eps;
    val_mixture = p_mixture./(std_sigma_mixture).^(-dist_param(1)-1);
    for q=1:regime
        z_t = repmat(A(:,q),1,regime_mixture);
        pa_t = repmat(pa(:,q),1,regime_mixture);
        w_t(:,q) = ((pa_t./s_mixture).*exp(z_t.*s_mixture))*val_mixture; %%% ./s_mixture comes from the gradient
    end
    w_t = w_t + 1e-18;
elseif(gamma_dist==2) %%% Log-Logistic
    dist_param = [sin(pi/gamma_param)*gamma_param/pi gamma_param];
    eps = (w_t./s);
    val_mixture = p_mixture./(std_sigma_mixture);
    for q=1:regime
        z_t = repmat(eps(:,q),1,regime_mixture);
        gradient = repmat(1./s(:,q),1,regime_mixture);
%         x_t = (z_t./(dist_param(1)*s_mixture));
%         aide = (gradient.*(dist_param(2)/dist_param(1)).*(x_t).^(dist_param(2)-1)./(1+(x_t).^(dist_param(2))).^2);
        
        x_t = (z_t./(s_mixture));
        x_t_pow = (x_t/dist_param(1)).^(dist_param(2));
        aide = dist_param(2).*gradient.*x_t_pow./(x_t.*(1+x_t_pow).^2);
%         aide = (gradient.*(dist_param(2)/dist_param(1)).*(x_t).^(dist_param(2)-1)./(1+(x_t).^(dist_param(2))).^2);
        w_t(:,q) = aide*val_mixture;
%         w_t(:,q) = (gradient.*(dist_param(2)/dist_param(1)).*(x_t).^(dist_param(2)-1)./(1+(x_t).^(dist_param(2))).^2)*val_mixture;
    end
    w_t = w_t + 1e-18;
else %%%% Gamma
    dist_param = [1/gamma_param gamma_param];
    eps = (w_t./s);
    pa = dist_param(2)^(-dist_param(1)).*(eps).^(dist_param(1)-1)./(s.*exp(gammaln(dist_param(1)))); %%% ./s comes from the gradient
    A = - eps./dist_param(2);
    val_mixture = p_mixture./(std_sigma_mixture).^(dist_param(1)-1);
    for q=1:regime
        z_t = repmat(A(:,q),1,regime_mixture);
        pa_t = repmat(pa(:,q),1,regime_mixture);
        w_t(:,q) = ((pa_t./s_mixture).*exp(z_t./s_mixture))*val_mixture; %%% ./s_mixture comes from the gradient
    end
    w_t = w_t + 1e-18;
end




for t=2:T+1          
    piA = (pi_mat(t-1,:)*P);
    
    
    
    C = (w_t(t-1,:).*piA); ft = sum(C);
    if ft == 0                      %This stop div by zero if probs are too low
        pi_mat(t,1) = 1;  
        disp('Log-likelihood is inf. Probably due to all zeros in pi_mat.')
    else
        pi_mat(t,:) = C / ft; 
    end
    
    LLs(t-1) = log(dot(w_t(t-1,:),piA));
end 
    LL=-sum(LLs);
    if ~isfinite(LL)
        disp('Log-likelihood is inf. Probably due to all zeros in pi_mat.')
    end
end



