/*********************************************************************
 * Demo.cpp
 *
 * This file shows the basics of setting up a mex file to work with
 * Matlab.  This example shows how to use 2D matricies.  This may
 * 
 * Keep in mind:
 * <> Use 0-based indexing as always in C or C++
 * <> Indexing is column-based as in Matlab (not row-based as in C)
 * <> Use linear indexing.  [x*dimy+y] instead of [x][y]
 *
 * For more information, see my site: www.shawnlankton.com
 * by: Shawn Lankton
 *
 ********************************************************************/
#include <matrix.h>
#include <mex.h>   
#include <math.h>
#include <float.h>

/* Definitions to keep compatibility with earlier versions of ML */
#ifndef MWSIZE_MAX
typedef int mwSize;
typedef int mwIndex;
typedef int mwSignedIndex;

#if (defined(_LP64) || defined(_WIN64)) && !defined(MX_COMPAT_32)
/* Currently 2^48 based on hardware limitations */
# define MWSIZE_MAX    281474976710655UL
# define MWINDEX_MAX   281474976710655UL
# define MWSINDEX_MAX  281474976710655L
# define MWSINDEX_MIN -281474976710655L
#else
# define MWSIZE_MAX    2147483647UL
# define MWINDEX_MAX   2147483647UL
# define MWSINDEX_MAX  2147483647L
# define MWSINDEX_MIN -2147483647L
#endif
#define MWSIZE_MIN    0UL
#define MWINDEX_MIN   0UL
#endif
/*  
  La fonction calcule le Forward-Backward d'un MSM model avec 3 tats differents dans la distribution M.
 *Pour l'appeler : 
 *[sn_sig log_like forward] = FB_sigma_c(eps_t,regime_sig,sigma,p_sig,Unif_rand)
 */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{

//declare variables
    mxArray *a_in_m,*b_in_m,*c_in_m,*d_in_m,*e_in_m,*f_in_m,*g_in_m,*h_in_m,*i_in_m,*j_in_m,*k_in_m,*l_in_m,*m_in_m,*prob_init_creation,*forward_creation,*back_trans_creation,*sn_current_creation,*sn_move_creation;
    const mwSize *dims;
    double *eps_t,eps_sq,*p_sig,*sigma,*sn_move,*prob_init,*Unif_rand,maxi,Q,sig_current,*forward_out,*forward,log_like,transition,*log_like_out,*back_trans,unif,*sigma_t_lev,*p_mixture,*sigma_mixture,like_mixt,*df,*CST;
    double eps_reg,temper,nb_replic_back;
    int  taille,taille_current,regime_sig,nb_iter_back,regime_mixture,stud_dist;
    int t,incr,i,z,q,j,taille_ARMA,index,ind;
    
    double PI = 3.141592;
    if (nrhs != 13) 
    {
        mexErrMsgTxt("13 inputs required.");
    } 
    else if (nlhs > 3) 
    {
        mexErrMsgTxt("Too many output arguments");
    }

//associate inputs
    a_in_m = mxDuplicateArray(prhs[0]);
    b_in_m = mxDuplicateArray(prhs[1]);
    c_in_m = mxDuplicateArray(prhs[2]);
    d_in_m = mxDuplicateArray(prhs[3]);
    e_in_m = mxDuplicateArray(prhs[4]);
    f_in_m = mxDuplicateArray(prhs[5]);
    g_in_m = mxDuplicateArray(prhs[6]);
    h_in_m = mxDuplicateArray(prhs[7]);
    i_in_m = mxDuplicateArray(prhs[8]);
    j_in_m = mxDuplicateArray(prhs[9]);
    k_in_m = mxDuplicateArray(prhs[10]);
    l_in_m = mxDuplicateArray(prhs[11]);
    m_in_m = mxDuplicateArray(prhs[12]);
    
    
    eps_t = mxGetPr(a_in_m);
    regime_sig = (int) (*mxGetPr(b_in_m));
    sigma = mxGetPr(c_in_m);
    p_sig = mxGetPr(d_in_m);
    Unif_rand = mxGetPr(e_in_m);
    temper =  (*mxGetPr(f_in_m));
    sigma_t_lev = mxGetPr(g_in_m);
    nb_replic_back = (*mxGetPr(h_in_m));
    sigma_mixture = mxGetPr(i_in_m);
    p_mixture = mxGetPr(j_in_m);
    regime_mixture  = (int) (*mxGetPr(k_in_m));
    df =  mxGetPr(l_in_m);
    CST = mxGetPr(m_in_m);
    
    if(df[0]==100)
    {
        stud_dist  = 0;
    }
    else
    {
        stud_dist  = 1;
    }

//figure out dimensions
    dims = mxGetDimensions(prhs[0]);
    taille = (int)dims[0]; 
    
    /*
     Creation des variables de sortie. On n'alloue la place sans en spcifier la valeur pour gagner du temps
    */
    if(nb_replic_back==0)
    {
        nb_iter_back = 1;
    }
    else
    {
        nb_iter_back = (int) nb_replic_back;
    }
    
    #define a_out_m  plhs[0]
    a_out_m = mxCreateDoubleMatrix(0,0,mxREAL);
    mxSetM(a_out_m,taille);
    mxSetN(a_out_m,nb_iter_back);
    mxSetData(a_out_m,mxMalloc(sizeof(double)*taille*nb_iter_back));
    sn_move = mxGetPr(a_out_m);
    
    
        
    
    #define c_out_m  plhs[2]
    c_out_m = mxCreateDoubleMatrix(0,0,mxREAL);
    mxSetM(c_out_m,taille);
    mxSetN(c_out_m,regime_sig);
    mxSetData(c_out_m,mxMalloc(sizeof(double)*taille*regime_sig));
    forward = mxGetPr(c_out_m);
    

    
    taille_current = taille;    
//     mexPrintf("Debut Forward");
    log_like = 0;
    transition = 0;
    for(t=0;t<taille;t++)
    { 
       //mexPrintf("Iteration %d",t); 
       maxi = -1e8;
       eps_sq = eps_t[t]*eps_t[t];
       for(i=0;i<regime_sig;i++)
       {
           if(t>0)
           {
               transition = 0;
               for(z=0;z<regime_sig;z++)
               {
                   
                   transition = transition + p_sig[z + i*regime_sig]*forward[(t-1) + z*taille_current];
                   
               }
           }
           else
           {
               transition = 1.0/regime_sig;//p_sig[0 + i*regime_sig];
           }
           
           
           if(transition!=0)
           {
               like_mixt = 0;
               if(stud_dist==1)
               {
                   for(z=0;z<regime_mixture;z++)
                   {
                       sig_current = sigma[i]*sigma_t_lev[t]*sigma_mixture[z];
                       like_mixt = like_mixt + exp(CST[0] -0.5*log(sig_current)-(0.5*(df[0]+1))*log(1+eps_sq/(df[0]*sig_current)))*p_mixture[z];
                   }
               }
               else
               {
                   for(z=0;z<regime_mixture;z++)
                   {
                       sig_current = sigma[i]*sigma_t_lev[t]*sigma_mixture[z];
                       like_mixt = like_mixt + exp(-0.5*log(2*PI*sig_current)-0.5*eps_sq/(sig_current))*p_mixture[z];
                   }
               }
               forward[t + taille_current*i] = log(like_mixt) + log(transition);
               if(maxi<forward[t + taille_current*i])
               {
                   maxi = forward[t + taille_current*i];
               }
                
//                 forward[t + taille_current*i] = exp(temper*(-0.5*log(2*PI*sig_current)-0.5*eps_sq/sig_current) + log(transition));
           }
           else
           {
               forward[t + taille_current*i] = 0;
           }
           
             
       }
       
       Q = 0; 
       for(i=0;i<regime_sig;i++)
       {
           if(forward[t + taille_current*i]!=0)
           {
               forward[t + taille_current*i] = exp(forward[t + taille_current*i]-maxi);
               Q = Q + forward[t + taille_current*i];
           }
       }
       
       for(i=0;i<regime_sig;i++)
       {
           forward[t + taille_current*i] = forward[t + taille_current*i]/Q;
//             mexPrintf("  forward[%d,%d] = %f",t + taille_current*i,i,forward[t + taille_current*i]);
       }
//        mexPrintf("\n");
//        for(i=0;i<regime_sig;i++)
//        {
//            mexPrintf("  Epsilon[%d,%d] = %f",t + taille_current*i,i,epsilon_Haas[(deb+t) + fin*i]);
//        }
//        mexPrintf("\n");
       log_like = log_like + log(Q) + maxi;
       
//        mexPrintf("log_dens = %f\n",log_like);
//        mexPrintf("maxi = %f\n",Q);
//             mexPrintf("theta[%d] = %f\n",1,theta[1]);
//             mexPrintf("theta[%d] = %f\n",2,theta[2]);
//             mexPrintf("theta[%d] = %f\n",3,theta[3]);
    }
    
    #define b_out_m  plhs[1]
    b_out_m = mxCreateDoubleScalar(log_like);
    log_like_out = mxGetPr(b_out_m);
    
    /*
      Procedure Backward 
    */
    /*
     Creation d'une variable. On n'alloue la place sans en spcifier la valeur pour gagner du temps
    */
    back_trans_creation = mxCreateDoubleMatrix(0,0,mxREAL);
    mxSetM(back_trans_creation,regime_sig);
    mxSetN(back_trans_creation,1);
    mxSetData(back_trans_creation,mxMalloc(sizeof(double)*regime_sig));
    back_trans = mxGetPr(back_trans_creation);
//     mexPrintf("Debut Back");
    for(j=0;j<nb_replic_back;j++)
    {
        ind = j*(taille);
        unif = Unif_rand[ind + taille-1];
        Q = forward[taille-1];
        i = 0;
        while(Q<unif) {
            i = i+1;
            Q = Q + forward[taille-1 + i*taille_current];
        }
        sn_move[ind + taille-1] = i;

        for(t=taille-2;t>=0;t--) {
            maxi = -1e9;
            q = (int) sn_move[ind + t+1];
    //         mexPrintf("   sn_move[%d] = %f   sn_stay = %f   ",t+1,sn_move[t+1],sn_current[t+1]);
            Q = 0;
            for(z=0;z<regime_sig;z++) 
            {

                back_trans[z] = forward[t + z*taille_current]*p_sig[z + q*regime_sig];

                Q = Q + back_trans[z];

            }
    //          mexPrintf("\n");
            for(i=0;i<regime_sig;i++) 
            {
                back_trans[i] = back_trans[i]/Q;
    //             mexPrintf("backward[%d,%d] = %f",taille_current-1-incr,i,back_trans[i]);
            }
    //         mexPrintf("\n");
            unif = Unif_rand[ind + t];
            if(Q!=0)
            {
            Q = back_trans[0];
            i = 0;
            while(Q<unif) {
                i = i+1;
                Q = Q + back_trans[i];
            }
            sn_move[ind + t] = i;
            }
            else
            {
                sn_move[ind + t] = 0;
            }


        }
    }
    


    return;
}
