function [GMM_estimate] = GMM_hierarchical_HMM_model(y,kbar_mixture,kbar,nb_free_MS,With_leverage,student_dist,estimation_mean,Simu_prev)
%dbstop if error;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% 
%%%%% kbar_mixture : number of components of the Main block
%%%%% kbar : number of components of the secondary block
%%%%% const_P : Number of parameters in the transition matrix (default = 4)
%%%%% (Possible values : 1,2,3,4 and 6 if kbar_mixture = 2)
%%%%% With_leverage : if =1 then leverage effect.
%%%%% student_dist : if =1 student distribution otherwise normal innovation
%%%%% Simu_prev : Structure with the starting point of the optimzation and the number of kbar (optional)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% To run : [Simu] = max_hierarchical_HMM_model(y,2,5,4,0,0);
%%%% Hierarchical MS model with 2 main components, kbar=5 (5 secondary components),
%%%% flexible P since const_P =4, no leverage, Normal dist..
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(nargin<8)
    Simu_prev = 0;
    if(nargin<7)
        estimation_mean = 0;
        if(nargin<6)
            student_dist = 0;
            if(nargin<5)
                With_leverage = 0;
                if(nargin<4)
                    nb_free_MS = 0;
                    if(nargin<3)
                        kbar = 5;
                        if(nargin<2)
                            kbar_mixture = 1;
                        end
                    end
                end
            end
        end
    end
end

const_P = 2+nb_free_MS;

if(isstruct(Simu_prev))
    if(isfield(Simu_prev,'theta'))
        init_theta = Simu_prev.theta;
        size_init_theta = max(size(init_theta));
        kbar_prev = Simu_prev.kbar;
    else
        size_init_theta = 0;
        kbar_prev = 0;
    end
else
    size_init_theta = 0;
    kbar_prev = 0;
end
% MSM_function = 0;
regime = 2^(kbar);
% regime_mixture = 2^kbar_mixture;




if(const_P<4 && const_P>0)
    nb_param_P = const_P;
else
   nb_param_P = kbar_mixture+kbar; 
   warning('const P should be between 1 and 6');
end

nb_param_lev = 0;
if(With_leverage~=0)
    nb_param_lev = 2;
end

if(student_dist==1)
    nb_stud = 1;
else
    nb_stud = 0;
end

nb_unc_mean = estimation_mean;
nb_param_sigma = 1+2+2+nb_free_MS;%1+1+kbar_mixture+2;


nb_param_tot = nb_param_P+nb_param_sigma+nb_param_lev+nb_stud+nb_unc_mean;
nb = 1;
if(size_init_theta==nb_param_tot)
    nb = 1;
end


disp('*******************************************************')
disp(['Estimation of the hierarchical MS model with ' num2str(kbar_mixture) ' Mixture components and ' num2str(kbar) ' Multifractal components'])
disp(['Nb parameters for the unc. mean : ' num2str(nb_unc_mean)])
disp(['Nb parameters for the variance : ' num2str(nb_param_sigma)])
disp(['Nb parameters for the transition matrix : ' num2str(nb_param_P)])
disp(['Nb parameters for the leverage effet : ' num2str(nb_param_lev)])
disp(['Nb parameters for the student distribution : ' num2str(nb_stud)])
disp('*******************************************************')
disp(['Total parameters : ' num2str(nb_param_tot)])
disp('*******************************************************')

nb_iter_dep = 50;
nb_chain = ceil(nb_param_tot*1.5);
param_chain = zeros(nb_param_tot,nb_chain);
GMM_chain = zeros(nb_chain,1);
outlier = ceil(0.2*nb_chain);
nb_chg = ceil(0.1*nb_iter_dep);

for i=1:nb
    test = 0;
    count = 0;
    if(i==1 && size_init_theta==nb_param_tot)
        theta_current = init_theta;
        GMM_val = moments_of_the_model(y,regime,kbar_mixture,theta_current,nb_param_sigma,With_leverage,student_dist,nb_unc_mean);
        if(GMM_val<1e9)
            test = 1;
        end
    end
    if(test==0)
        disp('Looking for a starting point...');
        for q=1:nb_chain
            for deb=1:10
                test = 0;
                while(test==0)
                    
                    theta_current = zeros(nb_param_tot,1);
                    theta_current(1) = mean(y);
                    
                    log_unc_var = log(var(y));
                    theta_current(nb_unc_mean+1) = randn() + log_unc_var;
                    theta_current(nb_unc_mean+2:nb_unc_mean+nb_param_sigma-2) = rand(2+nb_free_MS,1)*0.8;
                    theta_current(nb_unc_mean+2:nb_unc_mean+nb_param_sigma-2) = log(theta_current(nb_unc_mean+2:nb_unc_mean+nb_param_sigma-2));%./(1-theta_current(3:nb_param_sigma-2)));
                    theta_current(nb_unc_mean+nb_param_sigma-1:nb_unc_mean+nb_param_sigma) = rand(2,1)*0.8;
                    theta_current(nb_unc_mean+nb_param_sigma-1:nb_unc_mean+nb_param_sigma) = log(theta_current(nb_unc_mean+nb_param_sigma-1:nb_unc_mean+nb_param_sigma)./(1-theta_current(nb_unc_mean+nb_param_sigma-1:nb_unc_mean+nb_param_sigma)));
                    
                    
                    for z=1:nb_param_P
                        theta_current(nb_unc_mean+nb_param_sigma+z) = 1.5+randn();
                    end
                    if(nb_param_lev~=0)
                        if(nb_param_lev==2)
                            theta_current(nb_unc_mean+nb_param_sigma+nb_param_P+1) = 1.5+randn();
                            theta_current(nb_unc_mean+nb_param_sigma+nb_param_P+2) = -2+randn();
                        else
                            for z=1:nb_param_lev
                                theta_current(nb_unc_mean+nb_param_sigma+nb_param_P+z) = 1+randn();
                            end
                        end
                    end
                    if(student_dist==1)
                        theta_current(nb_unc_mean+nb_param_sigma+nb_param_P+nb_param_lev+1) = -2+randn();
                    end
                    
                    count = count +1;
                    GMM_val = moments_of_the_model(y,regime,kbar_mixture,theta_current,nb_param_sigma,With_leverage,student_dist,nb_unc_mean);
                    if(GMM_val<1e9)
                        test = 1;
                        
                    end
                    
                    
                end
                if(deb==1)
                    param_chain(:,q) = theta_current;
                    GMM_chain(q) = GMM_val;
                elseif(GMM_val<GMM_chain(q))
                    GMM_chain(q) = GMM_val;
                    param_chain(:,q) = theta_current;
                end
            end
        end
    end
    GMM_chain1 = GMM_chain;
    for q=1:nb_chain
        theta_current = param_chain(:,q);
        option_unc = optimset('MaxIter',10,'MaxFunEvals',100,'FinDiffType','central');
        [theta_current,GMM_val] = fminunc(@(theta) moments_of_the_model(y,regime,kbar_mixture,theta,nb_param_sigma,With_leverage,student_dist,nb_unc_mean),theta_current,option_unc);
        GMM_chain(q) = GMM_val;
        param_chain(:,q) = theta_current;
    end
    GMM_chain2 = GMM_chain;
    disp(['In process : ' num2str(0) ' %'])
    for k=1:nb_iter_dep
        
        for q=1:nb_chain
            [theta_prop] = DREAM(param_chain,q,0,nb_chain,k);
            GMM_val = moments_of_the_model(y,regime,kbar_mixture,theta_prop,nb_param_sigma,With_leverage,student_dist,nb_unc_mean);
            if(GMM_val<GMM_chain(q))
                GMM_chain(q) = GMM_val;
                param_chain(:,q) = theta_prop;
            end
        end
        
        if(mod(k,nb_chg)==0 && k<nb_iter_dep)
            disp(['In process : ' num2str(100*k/nb_iter_dep) ' %'])
            [~, indice] = sort(-1*GMM_chain);
            for z=1:outlier
                theta_current = param_chain(:,indice(end-outlier+z));
                option_unc = optimset('MaxIter',10,'MaxFunEvals',500,'FinDiffType','central');
                [theta_current,GMM_val] = fminunc(@(theta) moments_of_the_model(y,regime,kbar_mixture,theta,nb_param_sigma,With_leverage,student_dist,nb_unc_mean),theta_current,option_unc);
                GMM_chain(indice(z)) = GMM_val;
                param_chain(:,indice(z)) = theta_current;
                
            end
        end
    end
    disp([GMM_chain GMM_chain2 GMM_chain1])
    [GMM_val,ind] = min(GMM_chain);
    theta_current = param_chain(:,ind);
    
    disp(['Density of the starting point : ' num2str(-1*GMM_val)])
    
    val_theta = theta_current;
    if(kbar_prev==kbar)
        option_search = optimset('MaxIter',1e7,'MaxFunEvals',1e7,'Display','iter');
        [val_theta,GMM_val] = fminsearch(@(theta) moments_of_the_model(y,regime,kbar_mixture,theta,nb_param_sigma,With_leverage,student_dist,nb_unc_mean),val_theta,option_search);
%         options_pattern = psoptimset('Cache','on','Display','iter','TimeLimit',180,'TolMesh',1e-3);
%         [val_theta,GMM_val] = patternsearch(@(theta) -1*launch_FB_sigma_MLE_for_hierarchical_HMM(y,taille,regime,kbar_mixture,theta,nb_param_sigma,With_leverage,student_dist),val_theta,[],[],[],[],[],[],[],options_pattern);
    else
%         options_pattern = psoptimset('Cache','on','Display','iter','TolMesh',1e-6);
%         [val_theta] = patternsearch(@(theta) -1*launch_FB_sigma_MLE_for_hierarchical_HMM(y,taille,regime,kbar_mixture,theta,nb_param_sigma,With_leverage,student_dist),val_theta,[],[],[],[],[],[],[],options_pattern);
        option_unc = optimset('MaxIter',1e7,'MaxFunEvals',1e7,'Display','iter','FinDiffType','central');
        [val_theta] = fminunc(@(theta) moments_of_the_model(y,regime,kbar_mixture,theta,nb_param_sigma,With_leverage,student_dist,nb_unc_mean),val_theta,option_unc);
        option_search = optimset('MaxIter',10000,'MaxFunEvals',10000,'Display','iter');
        [val_theta,GMM_val] = fminsearch(@(theta) moments_of_the_model(y,regime,kbar_mixture,theta,nb_param_sigma,With_leverage,student_dist,nb_unc_mean),val_theta,option_search);
        
    end
     

    if(i==1)
        min_GMM_val = GMM_val;
        max_theta = val_theta;
        
    elseif(min_GMM_val>GMM_val)
       min_GMM_val = GMM_val;
       max_theta = val_theta;
    end
end
[~,GMM_estimate.Moments] = moments_of_the_model(y,regime,kbar_mixture,max_theta,nb_param_sigma,With_leverage,student_dist,nb_unc_mean);
GMM_estimate.theta = max_theta;
GMM_estimate.GMM_crit = min_GMM_val;
GMM_estimate.kbar = kbar;