function [model_parameter] = estimates_into_model_parameters(Simu,y)


theta = Simu.theta;
if(isfield(Simu,'nb_free_MS'))
    nb_free_MS = Simu.nb_free_MS;
else
    nb_free_MS = 0;
end
distribution_choice = Simu.distribution;
switch distribution_choice
    case 'Log-Logistic'
        gamma_dist = 2;
    case 'IGamma'
        gamma_dist = 0;
    otherwise
        gamma_dist = 1;
end

nb_param_sigma = 1+1+2+2+nb_free_MS;
if(gamma_dist==2)
    gamma_param = exp(theta(1))+1;
else
    gamma_param = exp(theta(1));
end

persistence = exp(theta(nb_param_sigma-1:nb_param_sigma))./(1+exp(theta(nb_param_sigma-1:nb_param_sigma)));
f_val = 1+exp(theta(3:4+nb_free_MS));%./(1+exp(theta(3:4+nb_free_MS)));%sort(1+exp(theta(3:3+kbar_main)));





if(Simu.kbar_leverage==0)
    model_parameter.theta_l = 0;
    model_parameter.l_1 = 0;
else
    model_parameter.theta_l = Simu.theta_lev(1);
    model_parameter.l_1 = Simu.theta_lev(2);
end


model_parameter.theta_c = persistence(end);
model_parameter.theta_m = persistence(1);
model_parameter.c_1 = f_val(end);
model_parameter.m_1 = f_val(1);


prob_mixt = Simu.theta_P(1);
model_parameter.prob_mixt_q = 1-prob_mixt;

%%%% mixture
f_mixt = [1 Simu.f_mixture];
p_mixt = Simu.P_mixture;
m0 = 1/(f_mixt*p_mixt);

f_mixt_corr = f_mixt*m0;
verif = f_mixt_corr*p_mixt;
disp(['Verif correction Mixt : ' num2str(verif)]);

%%%% MC
kbar = Simu.kbar;
f_MS = Simu.f_MS;
c0 = 1;
for q=1:kbar
    c0 = c0/(f_MS(q)/2+1/2);
end
f_MS_corr = f_MS*(c0)^(1/kbar);
verif = 1;
for q=1:kbar
    verif = verif*(f_MS_corr(q)/2+(c0)^(1/kbar)/2);
end
disp(['Verif correction MS : ' num2str(verif)]);

sigma_bar = exp(theta(2));
unc_var = sigma_bar/(m0*c0);



model_parameter.unc_var = unc_var;
model_parameter.f_MS_corr = f_MS_corr;
model_parameter.f_mixt_m0_times_mi = f_mixt_corr;

if(gamma_dist==1)
   kurt_eps = (gamma_param + 1); %%% Var(eta) = E(eta^2) -1;
else
    disp('Distribution pas encore faite');
    kurt_eps = gamma_param;
end

phi = zeros(kbar,1);
for q=1:kbar
    phi(q) = (model_parameter.theta_c^(q-1)*(model_parameter.c_1-1)/(2+model_parameter.theta_c^(q-1)*(model_parameter.c_1-1)))^2;
end

gamma = 2*Simu.theta_P(2)-1;

N_mixt = max(size(p_mixt));
m_i = zeros(N_mixt-1,1);
m_i(1) = model_parameter.m_1;
for q=2:N_mixt-1
   m_i(q) = 1 + model_parameter.theta_m*(m_i(q-1)-1); 
end
f_mixt_sq  = [1 m_i'].^2;

model_parameter.innovation_param = [1/gamma_param gamma_param];
model_parameter.distribution_choice = distribution_choice;

nb_corr = 200;
correl_th = zeros(nb_corr,1);
correl_C1 = zeros(nb_corr,1);
for k=1:nb_corr
    correl_th(k) = (prod(1+phi*(gamma^k)) -1)/(kurt_eps*prod(1+phi)*m0^2*(f_mixt_sq*p_mixt)-1);
    correl_C1(k) = gamma^k/(kurt_eps*prod(1+phi)*m0^2*f_mixt_sq*p_mixt-1);
end
model_parameter.correl_th = correl_th;
model_parameter.correl_1_MC = correl_C1;

[corr_emp] = autocorr(y,nb_corr);
model_parameter.correl_emp = corr_emp(2:end);

model_parameter.kurtosis = (kurt_eps*prod(1+phi)*m0^2*(f_mixt_sq*p_mixt)-1);

