function [DGP] = generate_process(Simu,taille)
gamma_param = exp(Simu.theta(1));
%%%% Probabilities
p_mixt = Simu.P_mixture;
P = Simu.P;

stat = P^500;
dist_stat = stat(1,:);
%%%% volatilities
sigma_bar = Simu.val_theta_sigma(1);
sigma = Simu.sigma/sigma_bar;
sigma_mixt = Simu.sigma_mixture;

burn = 4000;

%%%% Leverage
kbar_leverage = Simu.kbar_leverage;
fct_leverage = Simu.fct_leverage;

switch Simu.distribution
    case 'Gamma'
        gam_dist = 1;
        dist_param = [1/gamma_param gamma_param]; 
    otherwise
        gam_dist = 0;
        dist_param = [(gamma_param+1) gamma_param];
end

%%%% components
% regime_MS = max(size(sigma));
% regime_mixt = max(size(sigma_mixt));
% kbar = log(regime_MS)/log(2);
% kbar_mixt = log(regime_mixt)/log(2);
taille_y = max(size(Simu.y));
unc_var = zeros(taille_y,1);
rolling = 50;
for t=1:taille_y
   if(t<=rolling)
       unc_var(t) = var(Simu.y(:,2));
   else
       unc_var(t) = var(Simu.y(t-rolling:t,2));
   end
end
taille_all = taille+burn;
y = zeros(taille_all,1);
eps_return = zeros(taille_all,1);
leverage_t = ones(taille_all,1);
eps = zeros(taille_all,1);
st = multinomialrnd(dist_stat,rand());
st_mixt = multinomialrnd(p_mixt,rand());
vol = sigma_bar*sigma(st)*sigma_mixt(st_mixt);
var_t = zeros(taille_all,1);
if(gam_dist==1)
    eps(1) = gamrnd(dist_param(1),dist_param(2));
else
    eps(1) = 1/gamrnd(dist_param(1),1/dist_param(2));
end
model_variance = Simu.model_variance;
if(model_variance==1)
    y(1) = eps(1)*vol;
else
    y(1) = eps(1)*sqrt(vol);
end
leverage_effect = ones(kbar_leverage,1);
for t=2:taille_all
    st = multinomialrnd(P(st,:),rand());
    st_mixt = multinomialrnd(p_mixt,rand());
    leverage_t(t) = prod(leverage_effect);
    vol = sigma_bar*sigma(st)*sigma_mixt(st_mixt)*leverage_t(t);
    var_t(t) = vol;
    if(gam_dist==1)
        eps(t) = gamrnd(dist_param(1),dist_param(2));
    else
        eps(t) = 1/gamrnd(dist_param(1),1/dist_param(2));
    end
    if(model_variance==1)
        y(t) = eps(1)*vol;
    else
        y(t) = eps(1)*sqrt(vol);
    end
    if(kbar_leverage~=0)
        ind_unc_var = mod(t,taille_y)+1;
        eps_return(t) = randn()*sqrt(unc_var(ind_unc_var));
        ind_lev = kbar_leverage;
        dim_ind = 0;
        if(t<ind_lev)
            ind_lev = t;
            dim_ind = kbar_leverage-ind_lev;
        end
        eps_ind = eps_return(t:-1:t-ind_lev+1)<0;
        eps_ind = logical([eps_ind;zeros(dim_ind,1)]);
        
        window_eps_t = [eps_return(t:-1:t-ind_lev+1)./sqrt(leverage_t(t:-1:t-ind_lev+1));zeros(dim_ind,1)];
        leverage_effect = ones(kbar_leverage,1);
        leverage_effect(eps_ind) = 1+fct_leverage(eps_ind).*abs(window_eps_t(eps_ind));
    end
end
y = y(burn+1:end);
nb_corr = min(taille,200);
[corr_emp] = autocorr(y,nb_corr);
DGP.eps = eps;
DGP.corr_emp = corr_emp(2:end);
DGP.unc_var = unc_var;
DGP.y = y;
DGP.leverage_t = leverage_t(burn+1:end);
DGP.var_t = var_t(burn+1:end);

